// Copyright © Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.notification.provider.azure.pubsub;

import com.google.gson.JsonObject;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.opengroup.osdu.notification.provider.interfaces.IPubsubHandshakeHandler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;

@Component
public class EventGridHandshakeHandler implements IPubsubHandshakeHandler {

    private EventGridRequestBodyExtractor eventGridRequestBodyExtractor;

    private JaxRsDpsLog logger;

    @Autowired
    EventGridHandshakeHandler(JaxRsDpsLog logger, EventGridRequestBodyExtractor eventGridRequestBodyExtractor) {
        this.logger = logger;
        this.eventGridRequestBodyExtractor = eventGridRequestBodyExtractor;
    }

    /**
     * Extract Handshake response string form  Handshake request.
     * TODO: Check if there is a need to verify subscription name with
     *  registration service, before verifying the endpoint.
     * @return validation string
     */
    @Override
    public String getHandshakeResponse() {
        String validationCode = this.eventGridRequestBodyExtractor.getValidationCodeForHandshake();
        JsonObject jsonResponse = new JsonObject();
        jsonResponse.addProperty("ValidationResponse", validationCode);

        return jsonResponse.toString();
    }
}
